//===========================================================================
// Released under Anonymity
// Feel free to embrace and extend this code!
// 11/16/02
// 
// ResourceExtractorStatusOutput.h
// 
// Declaration of a class, an instance of which will be attached to a 
// ResourceExtractor instance to print status messages
//===========================================================================

//===========================================================================
// Include guard
//===========================================================================
#ifndef __RESOURCE_EXTRACTOR_STATUS_OUTPUT_H_INCLUDED
#define __RESOURCE_EXTRACTOR_STATUS_OUTPUT_H_INCLUDED

//===========================================================================
// Other included files
//===========================================================================
#include "projectpch.h"
#include "ResourceExtractorListener.h"

//===========================================================================
// Declaration of class: CResourceExtractorStatusOutput
//	implements IResourceExtractorListener
//
// CResourceExtractorStatusOuput implements the IResourceExtractorListener
// interface and thus receives status from a connected ResourceExtractor 
// object. Instances of this class print a marker every 50 Kilobytes 
// processed and print a status message for every 10 megabytes processed.  
// Also messages are printed on discovery and complete extraction of  
// any resources found.
//===========================================================================
class CResourceExtractorStatusOutput: public IResourceExtractorListener
{
private:
	//output
	//print a marker every this many bytes
	static const int c_printcountThreshold;

	//pring a status message every this many bytes
	static const int c_printcountThreshold2;
	
	//these variables hold the next threshold (always multiples of the above two variables
	size_t m_nextthreshold;
	size_t m_nextthreshold2;
	
	//whether or not we are extracting a resource.
	bool m_bExtracting;

	//the total number of bytes in the main file
	size_t m_totBytes;
	
	//the queue of bytes to be output.
	std::queue<char> m_queue;
	
	//a spinlock to protect access to the queue.
	CRITICAL_SECTION queueAccess;
	
	//Normally in the Reader/Writer problem a semaphore is used to track whether or not
	//the queue is full, however we don't need it because our effectively queue isn't 
	//bounded (love STL) so we only need to use the dataAvailable semaphore

	//a semaphore to count the number of data elements available
	HANDLE hDataAvailable;
	
	//the ThreadProc for the output thread:
	//input to the queue runs on whichever thread is running the resource extractor
	//to which this listener is attached.
	static void PrinterThread(void* pStatusOutput);

	// Handle of the thread doing the printing of the output.
	HANDLE m_hPrinterThread;

public:
	CResourceExtractorStatusOutput(void);
	~CResourceExtractorStatusOutput(void);

	// onByte gets called for every byte processed.
	void onByte(int byteValue,size_t byteNum);
	// Is called when the resource extractor opens a file
	void onFileOpen(char* szFilePath,size_t position);
	// This is called when the resource extractor closes a file
	void onFileClose(char* szFilePath, size_t position, size_t numBytes, size_t numTrailingBytes);
	// This is called once when the main data file is opened
	void onMainFileOpen(char* szMainFileName,size_t totalBytes);
	// Called once when the main data file is closed. 
	void onMainFileClose(char* szFileName);
	// Called when the listener is disconnected prematurely
	virtual void disconnect();
private:
	// prints a quick one character status marker.
	void printStatusMarker(void);
	// prints a message saying where you are in the big file
	void printProgressMessage(size_t byteNum);
	//puts one character onto the printqueue for the printer thread to take off.
	void produce(char c);
	//puts a series of characters onto the printqueue for the printer thread to take off.
	void produce(char* sz);
};

#endif